#' Projected ice dataset response
#'
#' \code{rerddap::griddap()} call to Projected ice dataset
#'
#' @format A list of class \code{griddap_nc} with elements:
#' \describe{
#'   \item{data}{a data frame of the extracted ice values}
#'   \item{summary}{metadata summary}
#' }
#' @source \url{https://coastwatch.noaa.gov/erddap/}
#' @docType data
#' @name proj_extract
"proj_extract"
