#' rerddapUtils: Miscellaneous utilities for the  'rerddap' package.
#'
#' The 'rerddapUtils' package is an 'R' package that is a set of four main
#' functions designed to work with and extend the 'rerddap' package. These functions includes
#' one for restricting by season, one for splitting large requests,
#' and two for working with projected datasets. There are also two utility functions
#' that provide estimates of the size of a proposed 'rerddap::griddap()' request.
#'
#'
#' @section Main Functions:
#' \itemize{
#' \item \code{griddap_season} - Restricts the request to a given season during the year
#' \item \code{griddap_season} - Splits a large extract into smaller pieces
#' \item \code{xy_to_latlon} - Converts projected coordinates from an 'rerddap::griddap()' request
#'  to latitude-longitude coordinates
#' \item \code{latlon_to_xy} - Converts latitude-longitude coordinates to projected coordinates.
#' \item \code{estimate_griddap_size} - Estimates the size of a proposed 'rerddap::griddap()' request.
#' \item \code{estimate_griddap_split_size} - Estimates the size of a proposed split 'rerddap::griddap()' request.
#' }
#' @section Details:
#' Besides the terse help documents,  more  detail for using the
#' functions are given  in the included vignette "using-rerddapUtils".
#'
#' @docType package
#' @name rerddapUtils
#' @keywords internal
"_PACKAGE"
## usethis namespace: start
## usethis namespace: end
NULL
