#' Projected 14 day wind info
#'
#' \code{rerddap::info()} call to 14 day wind dataset
#'
#' @format A list of class \code{info} with elements:
#' \describe{
#'   \item{variables}{a data frame with a list of the variables in the extraction}
#'   \item{alldata}{metadata summary}
#'   \item{base_url}{a character string givng the base URL of the extract}
#' }
#' @source \url{https://coastwatch.noaa.gov/erddap/}
#' @docType data
#' @name wind_info
"wind_info"
