% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wind_info.R
\docType{data}
\name{wind_info}
\alias{wind_info}
\title{Projected 14 day wind info}
\format{
A list of class \code{info} with elements:
\describe{
\item{variables}{a data frame with a list of the variables in the extraction}
\item{alldata}{metadata summary}
\item{base_url}{a character string givng the base URL of the extract}
}
}
\source{
\url{https://coastwatch.noaa.gov/erddap/}
}
\usage{
wind_info
}
\description{
\code{rerddap::info()} call to 14 day wind dataset
}
\keyword{datasets}
