% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proj_extract.R
\docType{data}
\name{proj_extract}
\alias{proj_extract}
\title{Projected ice dataset response}
\format{
A list of class \code{griddap_nc} with elements:
\describe{
\item{data}{a data frame of the extracted ice values}
\item{summary}{metadata summary}
}
}
\source{
\url{https://coastwatch.noaa.gov/erddap/}
}
\usage{
proj_extract
}
\description{
\code{rerddap::griddap()} call to Projected ice dataset
}
\keyword{datasets}
